/*
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

### File Specific:
   main-programmer: hdavid, amaechler
*/

#include "volfog_effect.h"

#include "util/loading/load_param.h"
#include "util/loading/factory.h"

#include "glincl.h"

#include "shell_command.h"

using namespace std;

CREATE_FACTORY(VolFogEffect, CL_VOLFOG_EFFECT);

VolFogEffect::VolFogEffect(const TiXmlElement* root)
{
  this->setClassID(CL_VOLFOG_EFFECT, "VolFogEffect");

  if (root != NULL)
    this->loadParams(root);

  this->init();

  //if (! this->init())                        // Check And Enable Fog Extension If Available
    //return false;                         // Return False If Extension Not Supported

  // Set Up Fog 
  //glEnable(GL_FOG);                         // Enable Fog
  //glFogi(GL_FOG_MODE, GL_LINEAR);                   // Fog Fade Is Linear
  //glFogfv(GL_FOG_COLOR, fogColor);                  // Set The Fog Color
  //glFogf(GL_FOG_START,  1.0f);                    // Set The Fog Start
  //glFogf(GL_FOG_END,    0.0f);                    // Set The Fog End
  //glHint(GL_FOG_HINT, GL_NICEST);                   // Per-Pixel Fog Calculation
  //glFogi(GL_FOG_COORDINATE_SOURCE_EXT, GL_FOG_COORDINATE_EXT);    // Set Fog Based On Vertice Coordinates

  this->activate();
}


VolFogEffect::~VolFogEffect()
{
  this->deactivate();
}

void VolFogEffect::loadParams(const TiXmlElement* root)
{
  WeatherEffect::loadParams(root);
}

bool VolFogEffect::init()
{
  PRINTF(0)("Initalize VolFogEffect\n");

  fogColor[0] = 0.6f;
  fogColor[1] = 0.3f;
  fogColor[2] = 0.0f;
  fogColor[3] = 1.0f;

  //glDepthFunc (GL_LEQUAL);                            // The Type Of Depth Testing
  //glEnable (GL_DEPTH_TEST);                           // Enable Depth Testing
  //glShadeModel (GL_SMOOTH);                           // Select Smooth Shading
  //glHint (GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST); // Set Perspective Calculations To Most AccurateAbility

  char extensions[ 16384 ];
  char Extension_Name[] = "GL_EXT_fog_coord";
  char *buf;

  sprintf( extensions, "%s", (char *)glGetString( GL_EXTENSIONS ) );
  buf = strtok( extensions, " " );
  while( buf != NULL )
  {
    if( !strcmp( Extension_Name, buf ) )
    {
      PRINTF(0)( "%s found, great.\n", Extension_Name );
      return true;
    }
    buf = strtok( NULL, " " );
  }

  PRINTF(0)( "%s\n", (char *)glGetString( GL_EXTENSIONS ) );
  PRINTF(0)( "%s not found, dammit.\n", Extension_Name );
  return false;
}


bool VolFogEffect::activate()
{
  PRINTF(0)("Activating VolFogEffect\n");
}

bool VolFogEffect::deactivate()
{
  PRINTF(0)("Deactivating VolFogEffect\n");
}


/**
 * draws the effect, if needed
 */
void VolFogEffect::draw() const
{
    glPushAttrib(GL_ENABLE_BIT);

glClearDepth (1.0f); 
  glDepthFunc (GL_LEQUAL);                            // The Type Of Depth Testing
  glEnable (GL_DEPTH_TEST);                           // Enable Depth Testing
  glShadeModel (GL_SMOOTH);                           // Select Smooth Shading
  glHint (GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST); // Set Perspective Calculations To Most AccurateAbili


  // Set Up Fog 
  glEnable(GL_FOG);                         // Enable Fog
  glFogi(GL_FOG_MODE, GL_LINEAR);                   // Fog Fade Is Linear
  glFogfv(GL_FOG_COLOR, fogColor);                  // Set The Fog Color
  glFogf(GL_FOG_START,  1.0f);                    // Set The Fog Start
  glFogf(GL_FOG_END,    0.0f);                    // Set The Fog End
  glHint(GL_FOG_HINT, GL_NICEST);                   // Per-Pixel Fog Calculation
  glFogi(GL_FOG_COORDINATE_SOURCE_EXT, GL_FOG_COORDINATE_EXT);    // Set Fog Based On Vertice Coordinates



  //glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);        // Clear Screen And Depth Buffer
  //glLoadIdentity ();                          // Reset The Modelview Matrix

  //glTranslatef(camx, 0.0f, camz);             // Move To Our Camera Z/X Position

  // glFogCoordEXT is very similar to glVertex3f. If you understand
  // the way vertexs are applied in OpenGL, you will not have any dificulty
  // understanding glFogCoordEXT.

  // In this tutorial we are applyng the fog in a corridor, so the fog
  // goes from the less density (the minor z) to a bigger density (the biggest z).
  // If you check the fog_start and fog_eng, it's 0 and 1.

  // So, we will pass to the function glFogCoordEXT, the fog value corresponding
  // with the glVertex3f value.If we are drawing a quad from z minus to z major, 
  // we do exactly the same with glFogCoord.

  // For example, in the first quad, is vertex coordinates in the Z field are all 
  // -15.0f. So we want the fog to completely fill this quad, so we assign 0 to all
  // the glFogCoordExt.
  
  glBegin(GL_QUADS);                          // Back Wall
    glFogCoordfEXT( 0.0f); glVertex3f(-2.5f,-2.5f,-15.0f);
    glFogCoordfEXT( 0.0f); glVertex3f( 2.5f,-2.5f,-15.0f);
    glFogCoordfEXT( 0.0f); glVertex3f( 2.5f, 2.5f,-15.0f);
    glFogCoordfEXT( 0.0f); glVertex3f(-2.5f, 2.5f,-15.0f);
  glEnd();

  glBegin(GL_QUADS);                          // Floor
    glFogCoordfEXT( 0.0f); glVertex3f(-2.5f,-2.5f,-15.0f);
    glFogCoordfEXT( 0.0f); glVertex3f( 2.5f,-2.5f,-15.0f);
    glFogCoordfEXT( 1.0f); glVertex3f( 2.5f,-2.5f, 15.0f);
    glFogCoordfEXT( 1.0f); glVertex3f(-2.5f,-2.5f, 15.0f);
  glEnd();

  glBegin(GL_QUADS);                          // Roof
    glFogCoordfEXT( 0.0f); glVertex3f(-2.5f, 2.5f,-15.0f);
    glFogCoordfEXT( 0.0f); glVertex3f( 2.5f, 2.5f,-15.0f);
    glFogCoordfEXT( 1.0f); glVertex3f( 2.5f, 2.5f, 15.0f);
    glFogCoordfEXT( 1.0f); glVertex3f(-2.5f, 2.5f, 15.0f);
  glEnd();

  glBegin(GL_QUADS);                          // Right Wall
    glFogCoordfEXT( 1.0f); glVertex3f( 2.5f,-2.5f, 15.0f);
    glFogCoordfEXT( 1.0f); glVertex3f( 2.5f, 2.5f, 15.0f);
    glFogCoordfEXT( 0.0f); glVertex3f( 2.5f, 2.5f,-15.0f);
    glFogCoordfEXT( 0.0f); glVertex3f( 2.5f,-2.5f,-15.0f);
  glEnd();

  glBegin(GL_QUADS);                          // Left Wall
    glFogCoordfEXT( 1.0f); glVertex3f(-2.5f,-2.5f, 15.0f);
    glFogCoordfEXT( 1.0f); glVertex3f(-2.5f, 2.5f, 15.0f);
    glFogCoordfEXT( 0.0f); glVertex3f(-2.5f, 2.5f,-15.0f);
    glFogCoordfEXT( 0.0f); glVertex3f(-2.5f,-2.5f,-15.0f);
  glEnd();
 
  //glFlush ();                             // Flush The GL Rendering Pipeline
  glPopAttrib();
}



/**
 * ticks the effect if there is any time dependancy
 */
void VolFogEffect::tick(float dt)	
{
}
